'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.keysToPathChildren = exports.keysToPath = exports.isLeaf = exports.asList = undefined;

var _immutable = require('immutable');

/**
 * @module utils
 */

/**
 * Accepts an Array, List or null and returns an equivalent List. Passing in null will return an empty List.
 *
 * @param {Array|List|null} input The input to be convert to a List. List items will pass through unchanged, all others will be passed into a List constructor.
 * @return {List} The equivalent List.
 */

function asList(input) {
  return _immutable.List.isList(input) ? input : (0, _immutable.List)(input);
}

/**
 * Accepts a node and returns a boolean indicating if the node is a leaf node (i.e. it has no children).
 *
 * @param {*} node The node to check.
 * @param {Array|List} [childPath=null] An {Array} or {List} of the key path to each node's children.
 * @return {boolean} A boolean indicating if the node is a leaf node.
 */

function isLeaf(node) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  if (!_immutable.Iterable.isIterable(node)) {
    return true;
  }
  var children = node.getIn(childPath || []);
  return !children || children.isEmpty();
}

/**
 * Turns a node's keys and its childPath into a full path.
 *
 * @param {Array|List} keys An {Array} or {List} of keys used to identify a node.
 * @param {Array|List} [childPath=null] An {Array} or {List} of the key path to each node's children.
 * @return {List}
 */

function keysToPath(keys) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  childPath = asList(childPath);
  return keys.reduce(function (fullPath, key) {
    return fullPath.concat(childPath).push(key);
  }, (0, _immutable.List)());
}

/**
 * Turns a node's keys and its childPath into a full path to the node's children.
 *
 * @param {Array|List} keys An {Array} or {List} of keys used to identify a node.
 * @param {Array|List} [childPath=null] An {Array} or {List} of the key path to each node's children.
 * @return {List}
 */

function keysToPathChildren(keys) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return childPath == null ? asList(keys) : keysToPath(keys, childPath).concat(childPath);
}

exports.asList = asList;
exports.isLeaf = isLeaf;
exports.keysToPath = keysToPath;
exports.keysToPathChildren = keysToPathChildren;